package org.example;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import org.example.common.utils.RandomFieldDataUtil;
import org.example.entity.ItemInfo;
import org.example.entity.OrderInfo;
import org.example.entity.UserInfo;
import org.example.service.ItemInfoService;
import org.example.service.OrderInfoService;
import org.example.service.UserInfoService;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

/**
 * 生成数据类
 */
@SpringBootTest
public class RandomDataTest {

    @Autowired
    private UserInfoService userInfoService;

    @Autowired
    private ItemInfoService itemInfoService;

    @Autowired
    private OrderInfoService orderInfoService;

    /**
     * 随机生成用户数据
     */
    @Test
    public void insertUserInfoData() {
        /**
         * 控制生成的数据条数
         */
        int number = 750;

        List<UserInfo> list = new ArrayList<>();
        int i = 0;
        int sum = 0;

        while (number-- > 0) {
            list.add(
                    new UserInfo()
                            .setUserName(RandomFieldDataUtil.getName())
                            .setAge(new Random().nextInt(91) + 10)
                            .setAddress(RandomFieldDataUtil.getProCity())
                            .setIp(RandomFieldDataUtil.getRandomIp())
                            .setIdCard(RandomFieldDataUtil.generateIdCard())
            );
            // 每1k条数据插入一次
            if (++i == 1000) {
                userInfoService.saveBatch(list);
                sum += list.size();
                System.out.println("插入" + sum + "条数据完成");
                i = 0;
                CollUtil.clear(list);
            }
        }
        if (list.size() > 0) {
            userInfoService.saveBatch(list);
        }
        System.out.println("插入数据完成");
    }

    /**
     * 根据用户表和商品表随机生成订单数据
     */
    @Test
    public void insertOrderData() {
        // 控制生成的数据条数
        int number = 9931;

        List<OrderInfo> list = new ArrayList<>();
        int i = 0;
        int sum = 0;
        // 随机获取数据库中的用户id和订单id
        List<Long> userIds = new ArrayList<>();
        List<Long> goodIds = new ArrayList<>();
        getUserIdsAndGoodIds(userIds, goodIds);
        while (number-- > 0) {

            Date orderDate = RandomFieldDataUtil.generateRandomDate();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy");
            String orderId = RandomFieldDataUtil.generateCode() + "-" + sdf.format(orderDate) + "-" + IdUtil.getSnowflake().nextId();

            int amount = new Random().nextInt(2951) + 50;
            list.add(
                    new OrderInfo()
                            .setOrderId(orderId)
                            .setUserId(userIds.get(new Random().nextInt(userIds.size())))
                            .setItemId(goodIds.get(new Random().nextInt(goodIds.size())))
                            .setQty(new Random().nextInt(15) + 1)
                            .setAmount(new BigDecimal(String.valueOf(amount)))
                            .setOrderDate(orderDate)
            );
            // 每1k条数据插入一次
            if (++i == 1000) {
                orderInfoService.saveBatch(list);
                sum += list.size();
                System.out.println("插入" + sum + "条数据完成");
                i = 0;
                CollUtil.clear(list);
                getUserIdsAndGoodIds(userIds, goodIds);
            }
        }
        if (list.size() > 0) {
            orderInfoService.saveBatch(list);
        }
        System.out.println("插入数据完成");
    }


    /**
     * 随机获取数据库中的用户id和商品id
     *
     * @param userIds
     * @param goodIds
     */
    private void getUserIdsAndGoodIds(List<Long> userIds, List<Long> goodIds) {
        CollUtil.clear(userIds);
        CollUtil.clear(goodIds);
        userIds.addAll(
                userInfoService.lambdaQuery().last("order by rand() limit 1000").list()
                        .stream().map(UserInfo::getUserId).collect(Collectors.toList())
        );
        goodIds.addAll(
                itemInfoService.lambdaQuery().last("order by rand() limit 1000").list()
                        .stream().map(ItemInfo::getItemId).collect(Collectors.toList())
        );
    }

}

