package org.example.task;

import cn.hutool.core.util.IdUtil;
import org.assertj.core.util.DateUtil;
import org.example.common.utils.RandomFieldDataUtil;
import org.example.entity.OrderInfo;
import org.example.service.ItemInfoService;
import org.example.service.OrderInfoService;
import org.example.service.UserInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Random;

/**
 * 固定时间提交订单
 */
@Component
public class IntervalCommitOrderTask {

    @Autowired
    private UserInfoService userInfoService;

    @Autowired
    private ItemInfoService itemInfoService;

    @Autowired
    private OrderInfoService orderInfoService;

    // 提交间隔，单位为毫秒
    private static final int SUBMIT_INTERVAL = 60 * 1000;

    @Scheduled(fixedRate = SUBMIT_INTERVAL)
    public void commitOrder() {
        Long userId = userInfoService.lambdaQuery().last("order by rand() limit 1").one().getUserId();
        Long itemId = itemInfoService.lambdaQuery().last("order by rand() limit 1").one().getItemId();

        Date orderDate = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy");
        String orderId = RandomFieldDataUtil.generateCode() + "-" + sdf.format(orderDate) + "-" + IdUtil.getSnowflake().nextId();
        int amount = new Random().nextInt(2951) + 50;
        orderInfoService.save(new OrderInfo()
                .setOrderId(orderId)
                .setUserId(userId)
                .setItemId(itemId)
                .setQty(new Random().nextInt(15) + 1)
                .setAmount(new BigDecimal(String.valueOf(amount)))
                .setOrderDate(orderDate)
        );
        System.out.println("提交订单：" + orderId +" 提交时间："+ DateUtil.now());
    }


}
